import inspect
from abc import ABC, abstractmethod
from typing import Dict, List, Union

from workspaces.system.schema import AbstractFile
from workspaces.workflow.schema_interfaces import (
    WorkflowIF,
    WorkflowRequestFileIF,
    WorkflowRequestIF,
)


class WorkflowServiceIF(ABC):
    """
    Executes workflows; should be a freestanding service.
    """

    @abstractmethod
    def execute(self, request: WorkflowRequestIF):
        """
        Execute this workflow against these files.

        :param request:
        :return:               a stream of events from this workflow
        """
        pass

    @abstractmethod
    def attach_file_to_request(self, request: WorkflowRequestIF, filename: str, content:bytes):
        pass

    @abstractmethod
    def create_workflow_request(self, workflow, argument) -> WorkflowRequestIF:
        pass


class WorkflowInfoIF(ABC):
    """
    Holds information about workflows.
    """

    @abstractmethod
    def lookup_workflow_definition(self, name: str) -> WorkflowIF:
        """
        Look up the workflow with this name.

        :param name:  Workflow name
        :return:      Workflow instance
        """
        pass

    @abstractmethod
    def lookup_workflow_request(self, request_id: int) -> WorkflowRequestIF:
        """
        Look up the workflow request with this ID.

        :param request_id: Request ID
        :return Workflow instance
        """
        pass

    @abstractmethod
    def all_workflows(self) -> List[WorkflowIF]:
        """
        Look up all the workflows that are defined by the system.

        :return all the workflows
        """
        pass

    @abstractmethod
    def create_workflow_request(
        self, workflow: Union[str, WorkflowIF], argument: Dict
    ) -> WorkflowRequestIF:
        """
        Create new workflow request and save to database
        :param workflow: workflow to run
        :param argument: workflow arguments
        :return: new WorkflowRequest
        """
        pass

    def save_request(self, request: WorkflowRequestIF):
        """
        Save a given request and return an integer identifier for it
        :param request: request to save
        :return:        the request id
        """
        pass

    def save_file(
        self, request: WorkflowRequestIF, filename: str, content: bytes
    ) -> WorkflowRequestFileIF:
        """
        Save a given file for the specified request to the database
        :param request: request to update with new file
        :param filename: filename of new file
        :param content: contents of new file in bytes
        :return:
        """
        pass
